#include "stdafx.h"

#include <stdio.h>

enum Direction {Horizontal=0,Vertical=1};

struct Point {
    int x,y;
};

struct PointList {
    Point *p;
    PointList *next;
} *X[2001],*Y[2001];

int NumPoints;

int minX=10000,maxX=-10000; // For optimizations (not needed, though - the program is fast enough

Point *Chain[15000];
int ChainSize;

int VisitedX[2001],VisitedY[2001];

FILE *ouf;

void ReadData()
{
    FILE *inf=fopen("COLOR.INP","rt");
    Point *p;
    PointList *l;

    fscanf(inf,"%d",&NumPoints);

    for(int i=0;i<NumPoints;i++) {
        p=new Point;

        fscanf(inf,"%d %d",&p->x,&p->y);
        p->x+=1000;
        p->y+=1000;

        l=new PointList;

        l->p=p;
        l->next=X[p->x];
        X[p->x]=l;

        l=new PointList;

        l->p=p;
        l->next=Y[p->y];
        Y[p->y]=l;

        if((p->x)<minX) minX=p->x;
        if((p->x)>maxX) maxX=p->x;
    }

    fclose(inf);
}

inline void PrintPoint(int &x,int &y,int &c)
{
    fprintf(ouf,"%d %d %d\n",x-1000,y-1000,c);
}

inline void FlipChain()
{
    register int i,j;
    Point *t;

    for(i=0,j=ChainSize-1;i<j;i++,j--) {
        t=Chain[i];
        Chain[i]=Chain[j];
        Chain[j]=t;
    }
}

inline void RemovePoint(int &x,int &y)
{
    PointList **l,*p;

    for(l=&X[x];(*l)->p->y!=y;l=&((*l)->next));

    p=(*l);
    (*l)=p->next;

    delete p;

    // We then remove it from the Y list, deleting the point itself
    for(l=&Y[y];(*l)->p->x!=x;l=&((*l)->next));

    p=(*l);
    (*l)=p->next;

    delete p->p;
    delete p;
}

// true - Whole chain was removed, false otherwise
inline bool ProcessCycle(Direction CycleDir,Direction &dir,int &x,int &y)
{
    int j,CycleStart,CurrColor;

    if(CycleDir==Horizontal) for(j=0;(Chain[j]->x!=x)||(Chain[j+1]->x==x);j++);
    else for(j=0;(Chain[j]->y!=y)||(Chain[j+1]->y==y);j++);

    CycleStart=j;

    // The points of the chain must have alternating colors
    CurrColor=0;

    for(;j<ChainSize;j++) {
        PrintPoint(x=Chain[j]->x,y=Chain[j]->y,CurrColor);
        CurrColor=!CurrColor;

        VisitedX[x]--;
        VisitedY[y]--;

        RemovePoint(x,y);
    }

    ChainSize=CycleStart;

    // Now we have removed the cycle

    // If there are no points in the chain - create another chain
    if(ChainSize==0) return true;

    // Otherwise process the last point of the chain
    x=Chain[ChainSize-1]->x;
    y=Chain[ChainSize-1]->y;

    if(ChainSize==1) {
        // We have only one point in the chain

        // If it is alone on the X
        if(!Y[y]->next) {
            // And alone on the Y
            if(!X[x]->next) {
                // Lone point, we can color it as we wish
                fprintf(ouf,"%d %d 0\n",x-1000,y-1000);

                VisitedX[x]--;
                VisitedY[y]--;

                RemovePoint(x,y);

                return true;
            }

            // Not alone on the Y
            dir=Vertical;
        }
        else if(!X[x]->next) {
            // Not alone on the X, but alone on the Y
            dir=Horizontal;
        }
        else {
            // Neighbors everywhere
            dir=Horizontal;
        }
    }
    else {
        // We have more than one points in the chain
        if(Chain[ChainSize-2]->x==Chain[ChainSize-1]->x) {
            // Last movement was vertical
            if(!Y[y]->next) {
                // And the point is alone on the X

                if(Chain[0]->x==Chain[1]->x) {
                    // First movement was vertical

                    if(!Y[Chain[0]->y]) {
                        // And the point is alone on the X

                        // We can remove the chain
                        CurrColor=0;

                        for(j=0;j<ChainSize;j++) {
                            PrintPoint(x=Chain[j]->x,y=Chain[j]->y,CurrColor);
                            CurrColor=!CurrColor;

                            VisitedX[x]--;
                            VisitedY[y]--;

                            RemovePoint(x,y);
                        }

                        ChainSize=0;

                        return true;
                    }
                    else {
                        // The point is not alone on the X

                        // We have to reverse the chain
                        FlipChain();

                        x=Chain[ChainSize-1]->x;
                        y=Chain[ChainSize-1]->y;

                        // Now the last movement has been vertical, so next is horizontal
                        dir=Horizontal;
                    }
                }
                else {
                    // First movement was horizontal

                    if(!X[Chain[0]->x]) {
                        // And the point is alone on the Y

                        // We can remove the chain
                        CurrColor=0;

                        for(j=0;j<ChainSize;j++) {
                            PrintPoint(x=Chain[j]->x,y=Chain[j]->y,CurrColor);
                            CurrColor=!CurrColor;

                            VisitedX[x]--;
                            VisitedY[y]--;

                            RemovePoint(x,y);
                        }

                        ChainSize=0;

                        return true;
                    }
                    else {
                        // The point is not alone on the Y

                        // We have to reverse the chain
                        FlipChain();

                        x=Chain[ChainSize-1]->x;
                        y=Chain[ChainSize-1]->y;

                        // Now the last movement has been horizontal, so next is vertical
                        dir=Vertical;
                    }
                }
            }
            else dir=Horizontal;
        }
        else {
            // Last movement was horizontal
            if(!X[x]->next) {
                // And the point is alone on the Y

                if(Chain[0]->x==Chain[1]->x) {
                    // First movement was vertical

                    if(!Y[Chain[0]->y]) {
                        // And the point is alone on the X

                        // We can remove the chain
                        CurrColor=0;

                        for(j=0;j<ChainSize;j++) {
                            PrintPoint(x=Chain[j]->x,y=Chain[j]->y,CurrColor);
                            CurrColor=!CurrColor;

                            VisitedX[x]--;
                            VisitedY[y]--;

                            RemovePoint(x,y);
                        }

                        ChainSize=0;

                        return true;
                    }
                    else {
                        // The point is not alone on the X

                        // We have to reverse the chain
                        FlipChain();

                        x=Chain[ChainSize-1]->x;
                        y=Chain[ChainSize-1]->y;

                        // Now the last movement has been vertical, so next is horizontal
                        dir=Horizontal;
                    }
                }
                else {
                    // First movement was horizontal

                    if(!X[Chain[0]->x]) {
                        // And the point is alone on the Y

                        // We can remove the chain
                        CurrColor=0;

                        for(j=0;j<ChainSize;j++) {
                            PrintPoint(x=Chain[j]->x,y=Chain[j]->y,CurrColor);
                            CurrColor=!CurrColor;

                            VisitedX[x]--;
                            VisitedY[y]--;

                            RemovePoint(x,y);
                        }

                        ChainSize=0;

                        return true;
                    }
                    else {
                        // The point is not alone on the Y

                        // We have to reverse the chain
                        FlipChain();

                        x=Chain[ChainSize-1]->x;
                        y=Chain[ChainSize-1]->y;

                        // Now the last movement has been horizontal, so next is vertical
                        dir=Vertical;
                    }
                }
            }
            else dir=Vertical;
        }
    }

    return false;
}

void Solve()
{
    int i=minX,j,x,y,CurrColor;

    Direction dir; // Horizontal or vertical direction

    while(true) {
        // First we get a random point
        for(;(i<=maxX)&&(!X[i]);i++);
        if(i>maxX) return;

        x=i;
        y=X[i]->p->y;

        // If it is alone on the X
        if(!Y[y]->next) {
            // And alone on the Y
            if(!X[x]->next) {
                // Lone point, we can color it as we wish
                fprintf(ouf,"%d %d 0\n",x-1000,y-1000);

                // And remove it from the list
                RemovePoint(x,y);

                continue;
            }

            // Not alone on the Y
            dir=Vertical;
        }
        else if(!X[x]->next) {
            // Not alone on the X, but alone on the Y
            dir=Horizontal;
        }
        else {
            // Neighbors everywhere
            dir=Horizontal;
        }

        Chain[0]=X[i]->p;
        ChainSize=1;

        VisitedX[x]=1;
        VisitedY[y]=1;

        while(true) {
            if(dir==Horizontal) {
                if(!Y[y]->next) {
                    // The point is alone on the X

                    if((!X[Chain[0]->x]->next)||(!Y[Chain[0]->y]->next)) {
                        // First point is alone on the X or Y, we can remove the chain
                        CurrColor=0;

                        for(j=0;j<ChainSize;j++) {
                            PrintPoint(x=Chain[j]->x,y=Chain[j]->y,CurrColor);
                            CurrColor=!CurrColor;

                            VisitedX[x]--;
                            VisitedY[y]--;

                            RemovePoint(x,y);
                        }

                        ChainSize=0;

                        break;
                    }
                    else {
                        // First point is not alone, we have to reverse the chain
                        FlipChain();

                        x=Chain[ChainSize-1]->x;
                        y=Chain[ChainSize-1]->y;

                        if(x==Chain[ChainSize-2]->x) {
                            // Now the last movement has been vertical, so next is horizontal
                            dir=Horizontal;
                        }
                        else {
                            // Now the last movement has been horizontal, so next is vertical
                            dir=Vertical;
                        }
                    }
                }
                else {
                    // Horizontal movement
                    if(Y[y]->p->x==x) Chain[ChainSize]=Y[y]->next->p;
                    else Chain[ChainSize]=Y[y]->p;

                    x=Chain[ChainSize++]->x;

                    VisitedX[x]++;
                    VisitedY[y]++;
            
                    if(VisitedX[x]>1) {
                        // We have a cycle in the chain
                        if(ProcessCycle(Horizontal,dir,x,y)) break;
                    }
                    else dir=Vertical;
                }
            }
            else {
                if(!X[x]->next) {
                    // The point is alone on the Y

                    if((!X[Chain[0]->x]->next)||(!Y[Chain[0]->y]->next)) {
                        // First point is alone on the X or Y, we can remove the chain
                        CurrColor=0;

                        for(j=0;j<ChainSize;j++) {
                            PrintPoint(x=Chain[j]->x,y=Chain[j]->y,CurrColor);
                            CurrColor=!CurrColor;

                            VisitedX[x]--;
                            VisitedY[y]--;

                            RemovePoint(x,y);
                        }

                        ChainSize=0;

                        break;
                    }
                    else {
                        // First point is not alone, we have to reverse the chain
                        FlipChain();

                        x=Chain[ChainSize-1]->x;
                        y=Chain[ChainSize-1]->y;

                        if(x==Chain[ChainSize-2]->x) {
                            // Now the last movement has been vertical, so next is horizontal
                            dir=Horizontal;
                        }
                        else {
                            // Now the last movement has been horizontal, so next is vertical
                            dir=Vertical;
                        }
                    }
                }
                else {
                    // Vertical movement
                    if(X[x]->p->y==y) Chain[ChainSize]=X[x]->next->p;
                    else Chain[ChainSize]=X[x]->p;

                    y=Chain[ChainSize++]->y;

                    VisitedX[x]++;
                    VisitedY[y]++;

                    if(VisitedY[y]>1) {
                        // We have a cycle in the chain
                        if(ProcessCycle(Vertical,dir,x,y)) break;
                    }
                    else dir=Horizontal;
                }
            }
        }
    }
}

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
    ReadData();

    ouf=fopen("COLOR.OUT","wt");

    Solve();

    fclose(ouf);

	return 0;
}



