#include <stdio.h>
#include <stdlib.h>

#define MAX_N 15000
#define MAX_ROWS 2001
#define MAX_COLS 2001

#define WHITE  0
#define BLACK  1
#define NOTDEF 2

typedef struct { int x, y; char color; int next[2], next_cnt; } point;

int n;
point points[MAX_N];
int rowColIdx[MAX_N], colRowIdx[MAX_N];

int rowColCmp( const void * a, const void * b )
{
   int * A = (int *) a, * B = (int *) b;
   int ay, by, ax, bx;

   ay = points[*A].y; ax = points[*A].x;
   by = points[*B].y; bx = points[*B].x;

   if ( ay < by ) return -1;
   if ( ay > by ) return 1;

   return ( ax < bx ) ? - 1 : ( ax > bx );
}

int colRowCmp( const void * a, const void * b )
{
   int * A = (int *) a, * B = (int *) b;
   int ay, by, ax, bx;

   ay = points[*A].y; ax = points[*A].x;
   by = points[*B].y; bx = points[*B].x;

   if ( ax < bx ) return -1;
   if ( ax > bx ) return 1;

   return ( ay < by ) ? - 1 : ( ay > by );
}

void makeSortIndexes( void )
{
   int i;
   
   for ( i = 0; i < n; i++ ) rowColIdx[i] = colRowIdx[i] = i;

   qsort( rowColIdx, n, sizeof(int), rowColCmp );
   qsort( colRowIdx, n, sizeof(int), colRowCmp );
}

void makeLinks( void )
{
   int i, x, t;

   for ( i = 0; i < n - 1; i++ )
   {
      x = rowColIdx[i];
      t = rowColIdx[i + 1];

      if ( points[x].y == points[t].y )
      {
         points[x].next[ points[x].next_cnt++ ] = t;
         points[t].next[ points[t].next_cnt++ ] = x;
         i++;
      }
   }

   for ( i = 0; i < n - 1; i++ )
   {
      x = colRowIdx[i];
      t = colRowIdx[i + 1];

      if ( points[x].x == points[t].x )
      {
         points[x].next[ points[x].next_cnt++ ] = t;
         points[t].next[ points[t].next_cnt++ ] = x;
         i++;
      }
   }   
}

void markLine( int start )
{
   int x, next; char color; int i;
   
   points[start].color = WHITE;

   for ( i = 0; i < 2; i++ )
   {
      x = start;
      color = points[x].color;

      for (;;)
      {
         if ( points[x].next_cnt == 0 ) break;
      
         next = points[x].next[0];
         if ( points[next].color != NOTDEF )
         {
            if ( points[x].next_cnt == 1 ) break;
            next = points[x].next[1];
         }

         if ( points[next].color != NOTDEF ) break;

         color = !color;
         points[next].color = color;

         x = next;
      }
   }
}

void markColors( void )
{
   int i;
   
   makeSortIndexes();
   makeLinks();

   for ( i = 0; i < n; i++ ) if ( points[i].color == NOTDEF )
   {
      if ( points[i].next_cnt == 0 )
         points[i].color = WHITE;
      else
         markLine(i);
   }
}

int main( void )
{
   int i, x, y;
   
   FILE * f = fopen( "color.inp", "r" );
   fscanf( f, "%d", &n );
   for ( i = 0; i < n; i++ )
   {
      fscanf( f, "%d %d", &x, &y );
      points[i].y = y + 1000;
      points[i].x = x + 1000;
      points[i].color = NOTDEF;
   }
   fclose(f);

   markColors();

   f = fopen( "color.out", "w" );
   if (n)
   {
      fprintf( f, "%d %d %d", points[0].x - 1000, points[0].y - 1000,
                              points[0].color );
   }
   
   for ( i = 1; i < n; i++ )
      fprintf( f, "\n%d %d %d", points[i].x - 1000, points[i].y - 1000,
                                points[i].color );
   fclose(f);
   
   return 0;
}
