 /***********************************************************
  *
  *   Program:    ColorX (NT objects against simple "Color")
  *   Created:    02/14/01 6:10:PM -- 02/15/01 9:52:PM (with no breaks!)
  *   Author:     V.Molotkov
  *   Requested:
  *   Comments:   4th test in 2001 Programmer's contest
	*								of PC Magazine(Bg)
  *
  ************************************************************
  *
  * History:
  *
  * $Log:$
  *
  ************************************************************/

//#include <stdio.h>
//#include <stdlib.h>

#pragma warning(disable: 4786)

#include <vector>
//#include <list>
#include <cassert>
#include <iostream>
#include <fstream>
#include <limits>
//#include <iomanip>
#include <algorithm>
#include <functional>
#include <numeric>
#include "timer.h"

using namespace std;

#define QUICKY
#define DEPTH2
#define QUICKY2//Quicky correction for DEPTH2
#define STATS

enum TESTS{NOTEST,TEST};
int verbose = 0;
TESTS test = NOTEST;

const int MAX_POINTS = 15000;
const int MAX_SIZE = 2001;
const int XY_SHIFT = 1000;

enum //Colors
{WHITE=-1,NOCOLOR,BLACK,UNKNOWN};

inline int sign(int x){return x>0?1:(x<0?-1:0);}

//*******************************************************
//typedef pair<int,int> Point;

//This is, in fact, pair<int,int>
//but with shorter names "x", "y"
//instead of "first" "second"
struct Point{
  int x;
  int y;
  Point():x(),y(){};
  Point(const int& x0, const int& y0):x(x0),y(y0){};
  Point(const Point& pt):x(pt.x),y(pt.y){};
};

//Without operations below VC++ refuses creating vector<Point>.
//They are needed for sorting this vector all the same
inline bool operator==(const Point& p, const Point& q)
	{return (p.x==q.x&&p.y==q.y);}
inline bool operator!=(const Point& p, const Point& q)
	{return !(p==q);}
inline bool operator<(const Point& p, const Point& q)
	{return (p.x<q.x||!(q.x<p.x)&&p.y<q.y);}
inline bool operator>(const Point& p, const Point& q)
	{return q<p;}
inline bool operator<=(const Point& p, const Point& q)
	{return !(p>q);}
inline bool operator>=(const Point& p, const Point& q)
	{return !(p<q);}

typedef vector<Point> Points;

ostream& operator<<(ostream& os,const Points& r){
	for(int i=0; i<r.size(); i++){
		os<<"("<<r[i].x<<","<<r[i].y<<")";
		if((i+1)%10==0)os<<endl;
		else os<<" ";
	}
	return os;
}

//*******************************************************
class ColorPoints{
public:
	ColorPoints(int n=0,int x=0,int y=0):rawX(x),rawY(y),
		points(n),zip_points(n),indX(x),indY(y),matrix(0),X(0),Y(0),
		nX(0),nY(0),colX(0),colY(0),
#ifdef STATS
		maxExcl(0),
#endif
		Color(0)//,colors(n)
		{points.clear();zip_points.clear();}

	//**********************************
	//Init from file
	void Init(ifstream& is){
		is>>N;
		points.reserve(N);
		rawX = rawY = MAX_SIZE;
		indX.resize(MAX_SIZE);
		indY.resize(MAX_SIZE);
		int x,y;
		for(int i=0;i<N;i++){
			is>>x>>y;
			points.push_back(Point(x+XY_SHIFT,y+XY_SHIFT));
			if(indX[x+XY_SHIFT]++==0)X++;
			if(indY[y+XY_SHIFT]++==0)Y++;
		}
		sort(points.begin(),points.end());
		matrix.resize(X*Y);
		nX.resize(X);
		nY.resize(Y);
		colX.resize(X);
		colY.resize(Y);
		if(verbose)
		cout<<endl<<"points="<<points.size()<<" X="<<X<<" Y="<<Y<<endl;
	}

	//**********************************
	//Init n random points inside a rectangle of size (x,y)
	int Init(int n,int x,int y, unsigned seed=1){
		points.reserve(n);
		rawX = x;
		rawY = y;
		indX.resize(x);
		indY.resize(y);
		N = 0;
		vector<int> fill(x*y);
		srand(seed);
		for(int i=0; i<50000; i++){
			int next = (double(rand())/double(RAND_MAX))*double(x*y-1);
			assert(0<=next%x&&next%x<x);
			assert(0<=next/x&&next/x<y);
			if(fill[next]==0){
				fill[next]++;
				points.push_back(Point(next%x,next/x));
				if(indX[next%x]++==0)X++;
				if(indY[next/x]++==0)Y++;
				if(++N==n)break;
			}
		}
		sort(points.begin(),points.end());
		matrix.resize(X*Y);
		nX.resize(X);
		nY.resize(Y);
		colX.resize(X);
		colY.resize(Y);
		return N;
	}

	//**********************************
	void Points2Matrix(){
		int row=-1, col=-1;//Not 0!
		int currX=-1, currY=-1;//Not 0!
		//incremental calculation of position of point in "compressed" matrix
		for(int i=0;i<points.size();i++){
			Point pt = points[i];
			if(pt.x>currX){//New row;
				row++;
				col = -1;
				currY=-1;
			}
			for(int y=currY+1;y<rawY;y++){
				if(y<=pt.y&&indY[y]>0) col++;
				if(y==pt.y) break;//col is found;
			}
			assert((Y*row+col)<Y*X);
			assert(col<Y);
			assert(row<X);
			matrix[Y*row+col] = UNKNOWN;
			currX = pt.x;
			currY = pt.y;
			zip_points.push_back(Point(row,col));
		}
	}

	//**********************************
	//Supposing that colors for the first i-1 points are set
	void SetColor(int i){
		Point pt = zip_points[i];
		int i0 = i;
		int ind = Y*pt.x+pt.y;
		if(i==0){
			matrix[ind]=WHITE;
			nX[pt.x]++;
			nY[pt.y]++;
			colX[pt.x] = WHITE;
			colY[pt.y] = WHITE;
			Color = WHITE;
			//colors[0] = WHITE;
			return;
		}
		vector<int> exclX(X),exclY(Y);
		int color;
		bool mismatch = false;
		int xX=0, xY=0;
		switch(colX[pt.x]){
		case NOCOLOR://The most easy
	 		if(colY[pt.y]==NOCOLOR)
	 			color = Color?-sign(Color):WHITE;//Any color is suitable
	 		else
	 			color = -colY[pt.y];
	 		break;
	 	default:
	 		if(colY[pt.y]==NOCOLOR)
	 			color = -colX[pt.x];
			else{//"Horizintal" color did not match with "vertical" one; choose vertical
	 			color = -colY[pt.y];
	 			mismatch = true;
			}
	 		break;
	 	}
		nX[pt.x]++;
		nY[pt.y]++;
	 	matrix[ind] = color;
		colX[pt.x] += color;
		colY[pt.y] += color;
		if(WHITE<=colX[pt.x]&&colX[pt.x]<=BLACK)
			mismatch = false;
		assert(WHITE-1<=colX[pt.x]);
		assert(colX[pt.x]<=BLACK+1);
		assert(WHITE<=colY[pt.y]);
		assert(colY[pt.y]<=BLACK);
		assert(WHITE==color||BLACK==color);
		Color += color;
	 	if(!mismatch) return;
		assert(colX[pt.x]==2*color);
#ifdef DEPTH2
		//Points except(0);
		vector<int> except(0);
#endif
		int row = pt.x;
		while(true){
			BEGIN:
			int col;
#ifdef QUICKY
	 		for(col=0;col<Y;col++)
	 			if(matrix[Y*row+col]==color&&exclY[col]==0&&colY[col]==color){
						matrix[Y*row+col] = -color;
						colX[row] -= 2*color;
						colY[col] -= 2*color;//This is to be changed as well! no swap!
																 //This dirty bug took me 5-6 hours to fix!
						goto END;
				}
#endif
	 		for(col=0;col<Y;col++){
	 			//Find in the current row the first element
	 			//with color == color:
				assert(WHITE<=colY[col]);
				assert(colY[col]<=BLACK);
	 			if(matrix[Y*row+col]==color&&exclY[col]==0){
#ifndef QUICKY
					if(colY[col]==color){//Simply change color, with no swap
						matrix[Y*row+col] = -color;
						colX[row] -= 2*color;
						colY[col] -= 2*color;//No color swap!
						//colors[i]=matrix[ind];//Bad: Ix86 has only one loop register
						goto END;
					}
#endif
	 				//Find in the current column the first element
	 				//with color opposite to color:
	 				for(int nextrow=pt.x;nextrow>=0;nextrow--){
	 					if(nextrow!=row&&matrix[Y*nextrow+col]==-color&&exclX[nextrow]==0)
						if(colX[nextrow]!=color){
							//swap the colors
							matrix[Y*row+col] = -color;
							colX[row] -= 2*color;
							assert(WHITE<=colX[row]&&colX[row]<=BLACK);
	 						matrix[Y*nextrow+col] = color;
							colX[nextrow] += 2*color;
							assert(WHITE-1<=colX[nextrow]&&colX[nextrow]<=BLACK+1);
	 						exclY[col] = 1;//Do not touch this col in the future
							xY++;
							break;
	 					}
#ifdef DEPTH2
						else
							except.push_back(nextrow);//Hoping that it will not be used
#endif
	 				}
					if(nextrow>=0){
						if(WHITE<=colX[nextrow]&&colX[nextrow]<=BLACK){
							//colors[i]=matrix[ind];//Bad
							goto END;
						}
	 					exclX[row] = 1;//Do not touch this row in the future
						xX++;
						row = nextrow;
						col = 0;
						goto BEGIN;//Who said "goto" is bad?
					}
				}
	 		}
			if(col==Y){//This may happen (with no bugs). E.g. N=20000, X==Y==400
#ifdef DEPTH2
//Make one more attempt to resolve the color contradictions
				if(SetColor2(pt.x,color,except))goto BEGIN;
#endif
				cout<<endl<<"The coloring algorithm failed"<<endl<<"for this configuration!"<<endl;
				//This was used in bug detection:
				if(verbose>0){
					cout<<endl<<"i="<<i
						 	<<" row0="<<pt.x<<" col0="<<pt.y<<" colX="<<colX[pt.x]
						 	<<" colY="<<colY[pt.y]<<endl
						 	<<"row="<<row<<" col="<<col<<" nX="<<nX[pt.x]<<endl
						 	<<"xX="<<xX<<" xY="<<xY<<" Color="<<Color
						 	<<endl<<endl;
				}
				if(verbose>1){
					for(row=0;row<=pt.x;row++){
						cout<<"colX["<<row<<"]="<<colX[row]
								<<" color("<<row<<","<<pt.y<<")=";
						if(matrix[Y*row+pt.y]!=NOCOLOR)
							cout<<matrix[Y*row+pt.y]<<endl;
						else
							cout<<0<<endl;
					}
				}
				if(verbose>2){
					int min = pt.y-20>-1?pt.y-20:0;
	 	 	 	 	cout<<endl<<"Fragment of the matrix:"<<endl<<endl;
	 	 	 	 	for(int i=0;i<=pt.x;i++){
	 	 	 	 		for(int j=min;j<=pt.y;j++)
	 	 	 	 			if(matrix[Y*i+j]==NOCOLOR||matrix[Y*i+j]==UNKNOWN)
	 	 	 	 				cout<<".";
	 	 	 	 			else
	 	 	 	 				cout<<(matrix[Y*i+j]==-1?"-":"+");
	 	 	 	 		cout<<endl;
	 	 	 	 	}
				}
				exit(1);
			}
		}//End while
		END:
#ifdef STATS
		if(maxExcl<xX+xY)maxExcl=xX+xY;
#endif
		return;
	}

#ifdef DEPTH2
	//**********************************
	bool SetColor2(int maxX,int color,vector<int>&except){
		for(int i=0;i<except.size();i++){
			int row = except[i];
			int col = 0;
#ifdef QUICKY2
			//Without this add on algorithm failed yet sometimes:
			//E.g. at random (seed==55) configuration with N=20000, X==Y==1000
			//By the way coloring of this config (+ IO) takes only about 6 sec
			//on my slow MMX233 with 32M RAM (and WinNT);
			//but if one changes X and Y to 2000, then2 min are needed already.
			//Of course, there are trivial optimizations (with much shorter
			//triple loops in SetColor), but there is almost no time left for this.
			//And I am much too tired as well (26 hours of marathon!)
			// -----------------------------------------------------------------
			// I believe at least that now (with QUICKY, QUICKY2 and DEPTH2
			// corrections), it can ALWAYS find the solution, if only it
			// exists (I failed to prove the existence theorem up to now).
			for(;col<Y;col++)
			if(matrix[Y*row+col]==color&&colX[row]==color&&colY[col]==color){
				//color is dominant on the row, so can be inverted without swap!
				matrix[Y*row+col] = -color;
				colX[row] -= 2*color;
				colY[col] -= 2*color;
				return true;
			}
#endif
			for(col=0;col<Y;col++)
 			for(int nextrow=maxX;nextrow>=0;nextrow--){
				if(matrix[Y*row+col]==color&&colX[row]==color)//color dominates in row
				if(matrix[Y*nextrow+col]==-color&&colX[nextrow]==-color){//-color dominates in nextrow
				//i.e. both colors are dominant on their rows
					//Then just swap these two colors:
					matrix[Y*row+col] = -color;
					matrix[Y*nextrow+col] = color;
					colX[row] -= 2*color;
					colX[nextrow] += 2*color;
					return true;
				}
			}
		}
		return false;
	}
#endif

	void SetColors(){
		for(int i=0;i<points.size();i++) SetColor(i);
	}

	void Print(ostream&os){
		int shift = (test==NOTEST)?XY_SHIFT:0;
		//for(int i=0;i<points.size();i++)
		int i=-1;
		for(int j=0;j<matrix.size();j++){
			if(matrix[j]!=NOCOLOR){
				i++;
				if(i==points.size())break;
				os<<points[i].x-shift<<" "<<points[i].y-shift<<" "
					//<<(colors[i]<0?0:1)<<endl;
					<<(matrix[j]<0?0:1)<<endl;
			}
		}
		//cout<<"i="<<i<<" points="<<points.size()<<endl;
	}

	//Members:
	int rawX,rawY,N;//Dimension of "raw" matrix of colored points and the number of points
	int X,Y;//Dimensions of "compressed" matrix (with empty rows&columns excluded)
//private:
	Points points,zip_points;//Array of "raw" points and of "compressed" points;
	//vector<int> colors;//Bad: Ix86 has only one loop register
	vector<int> indX,indY;
	//vector<bool> matrix;//Too exotic
	vector<int> matrix;//Here the main process will go;
	vector<int> colX,colY;//Auxiliary vectors;
	vector<int> nX,nY;//Were practically not used up to now
	int Color;
#ifdef STATS
	int maxExcl;//Excluded rows & columns in search process, to see the real depth of search
#endif
};

//************************************************
int main(int argc, char **argv)
{

	int N=200,X=20,Y=0;
	bool random = false;
	unsigned seed = 1;

	//Command line options for debug:
	for (int OptNum = 1; OptNum < argc; OptNum++)
	{
		string str = strupr(argv[OptNum]);
		string i_str = str.substr(1);

		switch(str[0]){
	 		case 'T' :
				test = TEST;
	 			break;
	 		case 'V' :
				if(str.size()==1)
					verbose = 1;
				else
					verbose = atoi(i_str.c_str());
	 			break;
	 		case 'N' :
				N = atoi(i_str.c_str());
	 			goto DEFAULT;
	 		case 'X' :
				X = atoi(i_str.c_str());
	 			goto DEFAULT;
	 		case 'Y' :
				Y = atoi(i_str.c_str());
	 			goto DEFAULT;
	 		case 'R' :
				random = true;
	 			goto DEFAULT;
	 		case 'S' :
				seed = atoi(i_str.c_str());
	 			goto DEFAULT;
			DEFAULT:
			default:
				test = TEST;
	 	}
	}
	if(Y==0)Y=X;
	if(random)
		seed = unsigned(::time(0));

	ColorPoints Pts;
	//Pts.Init(32,7,7);
	timer time;
	time_t curr_time = 0;
	if(verbose)
		cout<<"Init...";
	//Pts.Init(2000,100,100);//Failed due to dirty bug!
	//Pts.Init(200,20,20);//OK
	if(test==TEST)
		Pts.Init(N,X,Y,seed);
	else{
		ifstream is("COLOR.INP");
		if(!is){
			cout<<"Can not open COLOR.INP!"<<endl;
			exit(1);
		}
		Pts.Init(is);
		is.close();
	}

	if(verbose){
	 	time.fix();
	 	cout<<"Done! (";
	 	time.print(cout);
	 	cout<<")."<<endl<<endl;
	 	cout<<"N="<<Pts.N<<" X="<<Pts.X<<" Y="<<Pts.Y<<endl<<endl;
	 	//cout<<Pts.points<<endl<<endl;//OK?!
	 	cout<<"Points2Matrix...";
	}

	Pts.Points2Matrix();

	if(verbose){
	 	time.fix();
	 	cout<<"Done! (";
	 	time.print(cout);
	 	cout<<")."<<endl<<endl;
	 	cout<<"Setting colors...";
	}

	Pts.SetColors();

	if(verbose){
	 	time.fix();
	 	cout<<"Done! (";
	 	time.print(cout);
	 	cout<<")."<<endl<<endl;
#ifdef STATS
		if(verbose>1)
			cout<<"maxDepth="<<Pts.maxExcl<<endl<<endl;
#endif
	 	if(verbose>3||(verbose>0&&X<=64)){
	 		cout<<"Colored matrix (trivial rows&columns skipped):"<<endl<<endl;
	 	 	for(int i=0;i<Pts.X;i++){
	 	 		for(int j=0;j<Pts.Y;j++)
	 	 			if(Pts.matrix[Pts.Y*i+j]==NOCOLOR)
	 	 				cout<<".";
	 	 			else
	 	 				//cout<<(Pts.matrix[Y*i+j]==-1?0:1);
	 	 				cout<<(Pts.matrix[Pts.Y*i+j]==-1?"-":"+");
	 	 		cout<<endl;
	 	 	}
	 	}
	}
	ofstream os("COLOR.OUT");
	if(!os){
		cout<<"Can not open COLOR.OUT!"<<endl;
		exit(1);
	}
	Pts.Print(os);
	os.close();

	return 0;
}